/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>
#include <stdio.h>
#include <direct.h>

#include "outlook/OutlookConfig.h"
#include "outlook/winmaincpp.h" 
#include "outlook/ActionLog.h"

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spdm/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"

static BOOL dbg = false;

BSTR bstr;
SyncManager* syncManager;
SyncSource sourceContacts       = SyncSource(CONTACTS    );
SyncSource sourceAppointments   = SyncSource(APPOINTMENTS);
SyncSource sourceTasks          = SyncSource(TASKS       );
SyncSource sourceNotes          = SyncSource(NOTES       );



//****************************************************************************************************
/*
* return a pointer to new. free memory, please...
*/
wchar_t* getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) {
        //
        // This is abcxyz
        //
        goto finally;
    }
    p1 = p1+l+2;

    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }
   
    finally:

    return ret;

}

/*
* function to replace &lt; or &gt; to < or > from server response in syncML message. New function for SyncServer 2.0
*/
void replaceAmpLGt(string &s) {

    if (dbg) {
            MessageBox (NULL, s, TEXT("in replaceAmpLGt before"), MB_SETFOREGROUND |MB_OK); 
    }
    
    int position = 0;
    
    // search for &lt;
    position = pos(TEXT("&lt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT("<"), s, position);
        position = pos(TEXT("&lt;"), s);
    }
    // now search for &lt;
    position = 0;
    position = pos(TEXT("&gt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT(">"), s, position);
        position = pos(TEXT("&gt;"), s);
    }
    if (dbg) {
        MessageBox (NULL, s, TEXT("in replaceAmpLGt after"), MB_SETFOREGROUND |MB_OK);  
    }
    
}

/*
* Function to modify "&" with "&amp;" , "<" with "lt;" and ">" with "gt;" 
* to prevent server creash
*/

void encodeSpecialChar2XML(string &s) {
       
    int position = 0, previousPosition = 0;
       
    //
    // &
    //
    position = pos(TEXT("&"), s);
    while (position != -1) {        
        del(s, position, 1);
        ins(TEXT("&amp;"), s, position);
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("&"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // <
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("<"), s);
    while (position != -1) {        
        del(s, position, 1);
        ins(TEXT("&lt;"), s, position);
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("<"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // >
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT(">"), s);
    while (position != -1) {        
        del(s, position, 1);
        ins(TEXT("&gt;"), s, position);
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT(">"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    } 
       
}


wchar_t* encodeBase64UTF8(char* toEncode) {
    
    char*    base64   = NULL;
    int      len      = 0;    
    wchar_t* toReturn = NULL;
    wchar_t* tmp      = NULL;   
    
    len = strlen(toEncode);
    base64 = new char[2 * len + 1];
    encodeBase64(toEncode, base64, len);

    toReturn = char2wchar(base64);
      
    if (base64 != NULL)
        delete base64;
    
    return toReturn;    

}


/*
* Return the encoded wchar_t array.
*
* Function uses to encode wchar_t array into base64.
* 
* @param toEncode - the wchar_t array to encode
*/

wchar_t* encodeBase64W(wchar_t* toEncode) {

    char*    c        = NULL;
    char*    base64   = NULL;
    int      len      = 0;    
    wchar_t* toReturn = NULL;
    wchar_t* tmp      = NULL;
    
    //string s = string();
    //s += (wchar_t*)toEncode;
    // replaceCarriage(s);
    
    //tmp = (wchar_t*)s;

    c = wchar2char(toEncode);
    if (c == NULL) {
        return NULL;
    }
    
    len = strlen(c);
    base64 = new char[2 * len + 1];
    encodeBase64(c, base64, len);

    toReturn = char2wchar(base64);
    
    if (c != NULL)
        delete c;
    
    if (base64 != NULL)
        delete base64;
    
    return toReturn;    

}

wchar_t* decodeBase64WUTF8(wchar_t* toDecode) {
    
    int      num        = 0;
    byte*    reversed   = NULL;
    char*    content    = NULL;
    wchar_t* toReturn   = NULL;
    int k = 0;

    content = wchar2char(toDecode);
    
    if (content == NULL) {        
        return NULL;
    }    

    reversed = new byte[2 * strlen(content) + 1];   
    memset(reversed, 0, 2 * strlen(content) + 1);
    
    decodeBase64(content, reversed, &num);
    reversed[num] = 0;    
    
    k = MultiByteToWideChar(CP_UTF8, 0, (char*)reversed, -1, NULL, 0);    
    //
    // MultiByteToWideChar returns the number of byte required including the terminator too
    //
    toReturn = new wchar_t[k];
    wmemset(toReturn, 0, k);

    MultiByteToWideChar(CP_UTF8, 0, (char*)reversed, num, toReturn, k);

    if (content != NULL)
        delete content; 
    
    if (reversed != NULL)
        delete reversed; 
    
    return toReturn;    

}


/*
* Return the decoded wchar_t array.
*
* Function uses to dencode wchar_t array from base64.
* 
* @param toDecode - the wchar_t array to decode
*/

wchar_t* decodeBase64W(wchar_t* toDecode) {
    
    int      num        = 0;
    byte*    reversed   = NULL;
    char*    content    = NULL;
    wchar_t* toReturn   = NULL;
    
    content = wchar2char(toDecode);
    
    if (content == NULL) {        
        return NULL;
    }    

    reversed = new byte[2 * strlen(content) + 1];   
    memset(reversed, 0, 2 * strlen(content) + 1);
    
    decodeBase64(content, reversed, &num);
    reversed[num] = 0;
        
    toReturn = char2wchar((char*)reversed);

    if (content != NULL)
        delete content; 
    
    if (reversed != NULL)
        delete reversed; 
    
    return toReturn;    

}

void error() {
    //wsprintf(logmsg, TEXT("Error!\nerror code: %d\nerror message: %s"), lastErrorCode, lastErrorMsg);
    //LOG.error(logmsg);
}
// to return messages to user...
void error(string &errorMessage, int code, wchar_t* errMsg) {   
    wchar_t localTemp[DIM_LOCAL_TEMP];
    wsprintf(localTemp, TEXT("Error!\nerror code: %d\nerror message: %s"), code, errMsg);

    errorMessage = errorMessage + localTemp;
    ActionLOG.action((wchar_t*)errorMessage);
}
void error(string &errorMessage) {      
    error(errorMessage, lastErrorCode,lastErrorMsg);
}

void WINAPI printLog (LPCSTR log, LPCSTR level) {    

    int l = 0;
    int ret = 0;
    wchar_t* wlog = NULL;
    wchar_t wlevel [32]; 
    wlevel [0] = 0;
    
    l = strlen(log);

    wlog = new wchar_t[l + 1];
    MultiByteToWideChar (CP_ACP, 0, log, l, wlog, l);
    wlog[l] = 0;    
        
    l = strlen(level);
    MultiByteToWideChar (CP_ACP, 0, level, l, wlevel, l);
    wlevel[l] = 0;    
    
    if (wcscmp(wlevel, DEBUG_LEVEL) == 0)
        LOG.debug(wlog);
    else if (wcscmp(wlevel, INFO_LEVEL) == 0)
        LOG.info(wlog);
    else if (wcscmp(wlevel, ERROR_LEVEL) == 0)
        LOG.error(wlog);
    

    /*
    if (wcscmp(wlevel, DEBUG_LEVEL) == 0)
        ActionLOG.debug(wlog);
    else if (wcscmp(wlevel, INFO_LEVEL) == 0)
        ActionLOG.info(wlog);
    else if (wcscmp(wlevel, ERROR_LEVEL) == 0)
        ActionLOG.error(wlog);
    else
        ActionLOG.action(wlog);
    */
    if (wlog != NULL) {
        delete wlog;
    }
}


/*
* method used to complete array to send from client to server. It sets an all, new, mod and del items caming from
* client in the source for the server
*/

int completeArray(SyncSource &source, LPCSTR xml, wchar_t* typeOfArray) {
    
    ActionLOG.debug(TEXT("Enter method completeArray"));
    wchar_t localTemp[DIM_LOCAL_TEMP];
    wchar_t *wxml = NULL;   
    int l = strlen(xml);

    wxml = new wchar_t[l + 1];
    MultiByteToWideChar (CP_ACP, 0, xml, l, wxml, l);
    wxml[l] = 0;    

    wchar_t* element = NULL;
    wchar_t* tmp = NULL;
    wchar_t* id = NULL;
    wchar_t* messageXml = NULL;
    
    int i = 0;
    unsigned int pos = 0;   
    
    element = getElementContent(wxml, TEXT("numItems"), &pos);
    
    if (element == NULL) {
        if (dbg) {
            wsprintf(localTemp, TEXT("CompleteArray: numero di items %i"), l);
            MessageBox (NULL, localTemp, typeOfArray, MB_SETFOREGROUND |MB_OK);     
        }
        ActionLOG.debug(TEXT("Exit method completeArray: element numItems is NULL"));           
        return 0;
    }
        
    l = wcstol(element, &tmp, 0);
    
    SyncItem** arrayOfItems = new SyncItem* [l];
    
    if (dbg) {
        wsprintf(localTemp, TEXT("CompleteArray: numero di items %i"), l);
        MessageBox (NULL, localTemp, typeOfArray, MB_SETFOREGROUND |MB_OK);     
        wsprintf(localTemp, TEXT("lunghezza wxml: %i"), wcslen(wxml));
        MessageBox (NULL, localTemp, typeOfArray, MB_SETFOREGROUND |MB_OK);                     
        LOG.debug(wxml);
        MessageBox (NULL, wxml, typeOfArray, MB_SETFOREGROUND |MB_OK);  
    }
    ActionLOG.debug(TEXT("Method completeArray: before cicle for."));
    for (i = 0; i < l; i++) {
        unsigned int tmpPos = 0;
        wchar_t forDbg[256];
        wsprintf(forDbg, TEXT("Method completeArray: cicle for: number to-> %i, current index->%i "), l, i);
        ActionLOG.debug(forDbg);
        element = getElementContent(&wxml[pos], TEXT("row"), &tmpPos);
        
        pos = tmpPos + pos;

        id = getElementContent(element, TEXT("identifier"), NULL);
                
        string s = string();
        
        messageXml = getElementContent(element, TEXT("bodyMsgValue"), NULL);    

        arrayOfItems[i] = new SyncItem(id);     
        
        if (messageXml != NULL) {// it would be delete so don't setData
            
            string s = string();
                        
            /////// for UTF8
            
            s += TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            int len = 0;
            int k   = 0;
            char* messageXmlChar = NULL;
	        wchar_t* t = NULL;

            len = wcslen(messageXml);            
            
            //
            // calculate lenght of array
            //
            k = WideCharToMultiByte (CP_UTF8, 0, messageXml, len, messageXmlChar, 0, NULL, NULL);
            
            if (k != 0) {
                messageXmlChar = new char[k + 1];

                k = WideCharToMultiByte (CP_UTF8, 0, messageXml, len, messageXmlChar, k, 0, 0);	
	            
	            messageXmlChar[k] = 0;
                           
                t = encodeBase64UTF8(messageXmlChar);
                s = t;
            }

            if (messageXmlChar != NULL) {
                delete messageXmlChar;
            }
            
            /////
            
            // s = CDATA_START;
            // s += messageXml;
            // s += CDATA_END;
            /*
            s += TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            s += messageXml;
            // encodeSpecialChar2XML(s);

            // encode base64
            wchar_t* t = NULL;
            //t = encodeBase64W((wchar_t*)messageXml);
            t = encodeBase64W((wchar_t*)s);
            s = t;
            */
            if (dbg) {
                MessageBox (NULL, messageXml, TEXT ("messageXml"), MB_SETFOREGROUND |MB_OK);    
                MessageBox (NULL, s, TEXT ("string s"), MB_SETFOREGROUND |MB_OK);       
                wsprintf(localTemp, TEXT("lunghezza s: %i"), length(s));
                MessageBox (NULL, localTemp, typeOfArray, MB_SETFOREGROUND |MB_OK);     
                LOG.debug(s);
                }
            arrayOfItems[i]->setData((wchar_t*)s, ((length(s) + 1) *sizeof(wchar_t)));          
            //arrayOfItems[i]->setData(messageXml, (wcslen(messageXml)*sizeof(wchar_t)));           
            
            if (t != NULL)
                delete t;

        }
                
    }
    ActionLOG.debug(TEXT("Method completeArray: end cicle for"));
        if (element != NULL)
            delete element;
    if (messageXml != NULL)
            delete messageXml;
    if (wxml != NULL)
            delete wxml;
    if (id != NULL)
            delete id;
    
    ActionLOG.debug(TEXT("Method completeArray: end delete variables"));

    if (wcscmp(typeOfArray, ALL) == 0 && l > 0)
        source.setAllSyncItems(arrayOfItems, l);

    else if (wcscmp(typeOfArray, NEW) == 0 && l > 0) 
        source.setNewSyncItems(arrayOfItems, l);

    else if (wcscmp(typeOfArray, MOD) == 0 && l > 0)
        source.setUpdatedSyncItems(arrayOfItems, l);

    else if (wcscmp(typeOfArray,DEL) == 0 && l > 0)
        source.setDeletedSyncItems(arrayOfItems, l);
    
    ActionLOG.debug(TEXT("Exit Method completeArray"));
    return l;
}

/*
* set modified items for fast sync from client to server
*/

void setModifiedItems (SyncSource &source, int dataType, wchar_t* path, LPCSTR xmlNew, LPCSTR xmlMod, LPCSTR xmlDel) {

    
    ActionLOG.debug(TEXT("Enter method setModifiedItems."));
    if (dbg) {
        MessageBox (NULL, TEXT("FAST: in setModifiedItems... "), TEXT ("FAST: in setModifiedItems"), MB_SETFOREGROUND |MB_OK);  
    }
    
    wchar_t localTemp[DIM_LOCAL_TEMP];

    int l = completeArray(source, xmlNew, NEW);             
    
    wsprintf(localTemp, CLIENT_NEW_ITEMS, l); 
    ActionLOG.action(localTemp);

    l = completeArray(source, xmlMod, MOD); 
    
    wsprintf(localTemp, CLIENT_UPDATE_ITEMS, l); 
    ActionLOG.action(localTemp);

    l = completeArray(source, xmlDel, DEL); 
    
    wsprintf(localTemp, CLIENT_DELETE_ITEMS, l); 
    ActionLOG.action(localTemp);

    ActionLOG.debug(TEXT("Exit method setModifiedItems."));    
}


/*
*set all item for SLOW_SYNC from client to server
*/
void setAllItems(SyncSource &source, int dataType, wchar_t* path, LPCSTR xmlAll) {
            
    ActionLOG.debug(TEXT("Enter method setAllItems"));
    if (dbg) {
        MessageBox (NULL, TEXT("SLOW: in setAllItems... "), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
    }
        
    wchar_t localTemp[DIM_LOCAL_TEMP];

    int l = completeArray(source, xmlAll, ALL);
    
    wsprintf(localTemp, SET_ALL_ITEMS, l); 
    ActionLOG.action(localTemp);
        
    ActionLOG.debug(TEXT("Exit method setAllItems"));    

}

BOOL getFileList(wchar_t *dir, wchar_t** ptrArray, int *count, BOOL onlyFilesNumber) {

    WIN32_FIND_DATA FileData;
    wchar_t localTemp[256];
    HANDLE hFind;
    wchar_t toFind    [512];
    wchar_t szNewPath [512];
    wchar_t filterPattern [256];
    
    BOOL ret    = TRUE;
    szNewPath[0] =   0;
    int lung = 0;
    
    DWORD dwAttrs;
    BOOL fFinished = FALSE;
    
    wsprintf(filterPattern, TEXT("*.*")); 

    wsprintf(toFind, TEXT("%s\\%s"), dir, filterPattern);   
    
    if (dbg) {
        MessageBox (NULL, toFind, TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);
    }

    hFind = FindFirstFile(toFind, &FileData);
    
    int i = 0;

    if (hFind == INVALID_HANDLE_VALUE) {
        if (dbg)
		    MessageBox (NULL, TEXT("Invalid handle for retrieve files"), TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);
        return FALSE;
    }
    else {   

        if (onlyFilesNumber == TRUE)  {
            //only to retrieve number of files
	        while (!fFinished) {	
                wsprintf(localTemp, TEXT("%s\\%s"), dir, FileData.cFileName);
                if (dbg) {
		            MessageBox (NULL, localTemp, TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);
                }
                dwAttrs = GetFileAttributes(localTemp);
               
                if (dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
                   // nothing
                } else if (dwAttrs & FILE_ATTRIBUTE_HIDDEN) {
                    // nothing
                } else if (dwAttrs & FILE_ATTRIBUTE_SYSTEM) {
                    // nothing
                } else {
                    i++;                                        
                }
		        if (!FindNextFile(hFind, &FileData)) {
			        if (GetLastError() == ERROR_NO_MORE_FILES){
				        fFinished = TRUE;
			        } else {                          			   
				       // remain into loop
			        }
		        }
	        }
            if (dbg) {
                wsprintf(localTemp, TEXT("Numero di file %i"), i);
                MessageBox (NULL, localTemp, TEXT ("Totale number of files"), MB_SETFOREGROUND |MB_OK);
            }
            *count = i;
        }        
        else {

            while (!fFinished) {            
                wsprintf(szNewPath, TEXT("%s/%s"), dir, FileData.cFileName);
			    // wsprintf(szNewPath, TEXT("%s"), FileData.cFileName);
                dwAttrs = GetFileAttributes(szNewPath);
            
                 if (dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
                   // nothing
                } else if (dwAttrs & FILE_ATTRIBUTE_HIDDEN) {
                    // nothing
                } else if (dwAttrs & FILE_ATTRIBUTE_SYSTEM) {
                    // nothing
                } else {
                    ptrArray[i] = new wchar_t[256];
                    // wsprintf(ptrArray[i], szNewPath);
                    wsprintf(ptrArray[i], FileData.cFileName);
                    
                    i++;
                }     
            
                wsprintf(szNewPath, TEXT(""));
            			    
                if (!FindNextFile(hFind, &FileData)) {
				    if (GetLastError() == ERROR_NO_MORE_FILES){
					    fFinished = TRUE;
				    }
				    else {				
					    // remain into loop
				    }
			    }
                *count = i;
		    }
        }
    // Close the search handle.

	FindClose(hFind);
    }
    return ret;

}

wchar_t* readContentFromFile(wchar_t* path) {

	wchar_t wfilename [256];    
    wchar_t localTemp [256];
    int position = 0;
    int len = 0;
	wsprintf(wfilename, TEXT("%s"),  path);
	HANDLE file;
    DWORD lpFileSizeHigh;
    DWORD dwSize;
  
	wchar_t line[2048];
	FILE* f;
	wchar_t* ptr = NULL;
    
    if (dbg) {        
        MessageBox (NULL, TEXT("readContentFromFile"), TEXT ("readContentFromFile"), MB_SETFOREGROUND |MB_OK);
        MessageBox (NULL, path, TEXT ("readContentFromFile"), MB_SETFOREGROUND |MB_OK);
    }
	
    file = CreateFile(wfilename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    
    if( file ) {
        dwSize = GetFileSize(file, &lpFileSizeHigh);    
        ptr = new wchar_t[dwSize + 1];
        wmemset(ptr, 0, dwSize);
    } else {
        
        goto finally;
    }
    
    CloseHandle(file);

    f = _wfopen(path, TEXT("r"));
    
    if (f == NULL) {        
        goto finally;
    }
   
    while(fgetws(line, 2048, f) != NULL) {
        
        len = wcslen(line);
        wcsncpy(&ptr[position], line, len);
       
        position = position + len;	   
        
    }

    fflush(f);
    fclose(f);

finally:

    return ptr;        

}


int completeArraySetAllItemFromFileSystem(SyncSource &source, wchar_t* path, wchar_t** ptrArray, int elementNumber) {           
    
    SyncItem** arrayOfItems = new SyncItem* [elementNumber];
    
    wchar_t localTemp[512];
    wsprintf(localTemp, TEXT("ElementNumber %i"), elementNumber);
    if (dbg) {
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
    }

    for (int i = 0; i < elementNumber; i++) {
       
        unsigned int tmpPos = 0;
        wchar_t* fileContent = NULL;
        wchar_t completePath [512];
        
        wsprintf(completePath, TEXT("%s%s\\%s"), path, DIR_ALL_ITEM_CONTACT, ptrArray[i]);                        
        if (dbg) {
            MessageBox (NULL, path, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
            MessageBox (NULL, ptrArray[i], TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
            MessageBox (NULL, completePath, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
        }
        
        arrayOfItems[i] = new SyncItem(ptrArray[i]);     
        
        fileContent = readContentFromFile(completePath);
        
        if (dbg) {
            MessageBox (NULL, fileContent, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
                              
        }

        if (fileContent != NULL) {// it would be delete so don't setData
            
            wchar_t* t = NULL;
            t = encodeBase64W(fileContent);
            arrayOfItems[i]->setData(t, ((wcslen(t) + 1) *sizeof(wchar_t)));          
            
            if (t != NULL)
                delete t;

        }

        if (fileContent != NULL) {
            delete[] fileContent;
        }

                
    }
    
    source.setAllSyncItems(arrayOfItems, elementNumber);

  
    return elementNumber;
}



/*
*set all item for SLOW_SYNC from client to server
*/
void setAllItemsFromFileSystem(SyncSource &source, wchar_t* path) {
            
    ActionLOG.debug(TEXT("Enter method setAllItems"));
    if (dbg) {
        MessageBox (NULL, TEXT("SLOW: in setAllItemsFromFileSystem... "), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
    }
        
    wchar_t localTemp[DIM_LOCAL_TEMP];
    wchar_t completePath[DIM_LOCAL_TEMP];
    wchar_t** ptrArray;
    int count = 0;

    // 
    //
    // get number of file in dir
    wsprintf(completePath, TEXT("%s%s"), path, DIR_ALL_ITEM_CONTACT);
    
    if (dbg) {
        MessageBox (NULL, completePath, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
    }

    getFileList(completePath, NULL, &count, TRUE);

    wsprintf(localTemp, TEXT("%i"), count);

    ptrArray = new wchar_t*[count];
    
    getFileList(completePath, ptrArray, &count, FALSE);

    wsprintf(localTemp, TEXT("%i"), count);
    if (dbg) {
        MessageBox (NULL, TEXT("After getFileList with complete... "), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);                       
    }

    int l = completeArraySetAllItemFromFileSystem(source, path, ptrArray, count);         

}



/*
* From server to client outlook. NEW ITEMS
*/
void manageNewItems(SyncItem** items, int n, int dataType, string &newString) {
    
    ActionLOG.debug(TEXT("Enter Method manageNewItems."));
    wchar_t dummy [DIM_LOCAL_TEMP];
    wchar_t localTemp [DIM_LOCAL_TEMP];
    int i = 0;
    
    wsprintf(localTemp, TEXT("%i"), n);     
    newString = TEXT("");

    if (n > 0) {
            newString = newString + TEXT("<NEW>");
            newString = newString + TEXT("<numItems>") + localTemp + TEXT("</numItems>");
    }
        
    for (i = 0; i < n; i++) {
            
        newString = newString + TEXT("<row>");

        items[i]->getKey(dummy);
        newString = newString + TEXT("<identifier>") + dummy + TEXT("</identifier>");
        newString = newString + TEXT("<bodyMsgValue>");
        
        wchar_t * ptrData = NULL;
        ptrData = (wchar_t*) items[i]->getData();
        // ptrData is like <contact><firstName>John</firstName></contact>
        
        // decode b64
        wchar_t* t = NULL;
        // t = decodeBase64W(ptrData);
        t = decodeBase64WUTF8(ptrData);
        delete ptrData;    
        ptrData = t;

        newString = newString + ptrData;
        replaceAmpLGt(newString);

        newString = newString + TEXT("</bodyMsgValue>");                        
        newString = newString + TEXT("</row>");
    }
        
    if (n > 0) {    
        newString = newString + TEXT("</NEW>");
    }
    ActionLOG.debug(TEXT("Exit Method manageNewItems."));
}


/*
* From server to client outlook. UPDATED ITEMS
*/

void manageUpdatedItems(SyncItem** items, int n, int dataType, string &modString) {
    
    ActionLOG.debug(TEXT("Enter Method manageUpdatedItems."));
    wchar_t dummy [DIM_LOCAL_TEMP];
    wchar_t localTemp [DIM_LOCAL_TEMP];
    int i = 0;
    
    wsprintf(localTemp, TEXT("%i"), n);
    
    modString = TEXT("");

    if (n > 0) {
        modString = modString + TEXT("<MOD>");
        modString = modString + TEXT("<numItems>") + localTemp + TEXT("</numItems>");
    }
        
    for (i = 0; i < n; i++) {

        modString = modString + TEXT("<row>");
        
        items[i]->getKey(dummy);
        modString = modString + TEXT("<identifier>") + dummy + TEXT("</identifier>");
        modString = modString + TEXT("<bodyMsgValue>");
                
        wchar_t * ptrData = NULL;
        ptrData = (wchar_t*) items[i]->getData();
        
        // decode b64
        wchar_t* t = NULL;
        //t = decodeBase64W(ptrData);
        t = decodeBase64WUTF8(ptrData);
        delete ptrData;    
        ptrData = t;

        modString = modString + ptrData;
        replaceAmpLGt(modString);

        modString = modString + TEXT("</bodyMsgValue>");                        
        modString = modString + TEXT("</row>");
    }
    
    if (n > 0) {    
        modString = modString + TEXT("</MOD>");
    }               
    ActionLOG.debug(TEXT("Exit Method manageUpdatedItems."));
}

/*
* From server to client outlook
*/

void manageDeletedItems(SyncItem** items, int n, int dataType, string &delString) {
    
    ActionLOG.debug(TEXT("Enter Method manageUpdatedItems."));
    wchar_t dummy [DIM_LOCAL_TEMP];
    wchar_t localTemp [DIM_LOCAL_TEMP];
    int i = 0;
    
    wsprintf(localTemp, TEXT("%i"), n);
    
    delString = TEXT("");

    if (n > 0) {
            delString = delString + TEXT("<DEL>");
            delString = delString + TEXT("<numItems>") + localTemp + TEXT("</numItems>");
    }
            
    for (i = 0; i < n; i++) {

        delString = delString + TEXT("<row>");
        
        items[i]->getKey(dummy);
        delString = delString + TEXT("<identifier>") + dummy + TEXT("</identifier>");                                   
        delString = delString + TEXT("</row>");
    }

    if (n > 0) {
        delString = delString + TEXT("</DEL>");
    }
    ActionLOG.debug(TEXT("Exit Method manageUpdatedItems."));
}

/*
* From server to client outlook. Managed returned items calling right methods...
*/
void manageReturnedItems(SyncSource& s, int dataType, wchar_t* path, string &completeMessage) {
    
    ActionLOG.debug(TEXT("Enter Method manageReturnedItems."));
    unsigned int n, i = 0;
    SyncItem** items;   
    wchar_t localTemp [DIM_LOCAL_TEMP];

    ActionLOG.action(MESSAGE_FROM_SERVER);
    ActionLOG.debug(TEXT("Method manageReturnedItems: start analysis new items"));
        // ************** NEW **********************
    
    n = s.getNewSyncItemsCount();
    items = s.getNewSyncItems();

    string returnNew = string();

    wsprintf(localTemp,SERVER_NEW_ITEMS, n);
    ActionLOG.action(localTemp);

    if (dbg) {      
        MessageBox (NULL, localTemp, TEXT ("Return New: localTemp"), MB_SETFOREGROUND |MB_OK);
    }
                                        
    manageNewItems(items, n, dataType, returnNew);

    if (dbg) {
        MessageBox (NULL, returnNew, TEXT ("ReturnNew: returnNew "), MB_SETFOREGROUND |MB_OK);
    }
    ActionLOG.debug(TEXT("Method manageReturnedItems: end analysis new items"));
    ActionLOG.debug(TEXT("Method manageReturnedItems: start analysis updated items"));
        // ************** UPDATED *******************
        
    n = s.getUpdatedSyncItemsCount();
    items = s.getUpdatedSyncItems();
        
    string returnModified = string();

    wsprintf(localTemp,SERVER_UPDATE_ITEMS, n);
    ActionLOG.action(localTemp);

    if (dbg) {      
        MessageBox (NULL, localTemp, TEXT ("Update localtemp"), MB_SETFOREGROUND |MB_OK);
    }
                                        
    manageUpdatedItems(items, n, dataType, returnModified);

    if (dbg) {
        MessageBox (NULL, returnModified, TEXT ("Update"), MB_SETFOREGROUND |MB_OK);
    }
    ActionLOG.debug(TEXT("Method manageReturnedItems: end analysis updated items"));
    ActionLOG.debug(TEXT("Method manageReturnedItems: start analysis deleted items"));
        // ************** DELETED ********************
    
    n = s.getDeletedSyncItemsCount();
    items = s.getDeletedSyncItems();

    string returnDeleted = string();

    wsprintf(localTemp,SERVER_DELETE_ITEMS, n );
    ActionLOG.action(localTemp);

    if (dbg) {

        MessageBox (NULL, localTemp, TEXT ("delete localtemp"), MB_SETFOREGROUND |MB_OK);
    }
   
    manageDeletedItems(items, n, dataType, returnDeleted);

    if (dbg) {
        MessageBox (NULL, returnDeleted, TEXT ("returnedDeleted"), MB_SETFOREGROUND |MB_OK);
    }
    ActionLOG.debug(TEXT("Method manageReturnedItems: end analysis deleted items"));
        
    completeMessage = completeMessage + TEXT("<xmlMessage>");
    completeMessage = completeMessage + returnNew;
    completeMessage = completeMessage + returnModified;
    completeMessage = completeMessage + returnDeleted;
    completeMessage = completeMessage + TEXT("</xmlMessage>");

    if (dbg) {
        MessageBox (NULL, (wchar_t*)completeMessage, TEXT ("completeMessage"), MB_SETFOREGROUND |MB_OK);
    }
    ActionLOG.debug(TEXT("Exit Method manageReturnedItems"));

}



void deleteObject(wchar_t* sourceName) {

    SyncItem **p1;
    SyncItem **p2;
    SyncItem **p3;
    SyncItem **p4;
    
    int newCount, modCount, delCount, allCount;
    
    if (wcscmp(CONTACTS, sourceName) == 0) {
        p1 = sourceContacts.getNewSyncItems    ();
        p2 = sourceContacts.getUpdatedSyncItems();
        p3 = sourceContacts.getDeletedSyncItems();
        p4 = sourceContacts.getAllSyncItems    ();
        
        newCount = sourceContacts.getNewSyncItemsCount();
        modCount = sourceContacts.getUpdatedSyncItemsCount();
        delCount = sourceContacts.getDeletedSyncItemsCount();
        allCount = sourceContacts.getAllSyncItemsCount();    
        
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0) {
        p1 = sourceAppointments.getNewSyncItems    ();
        p2 = sourceAppointments.getUpdatedSyncItems();
        p3 = sourceAppointments.getDeletedSyncItems();
        p4 = sourceAppointments.getAllSyncItems    ();
        
        newCount = sourceAppointments.getNewSyncItemsCount();
        modCount = sourceAppointments.getUpdatedSyncItemsCount();
        delCount = sourceAppointments.getDeletedSyncItemsCount();
        allCount = sourceAppointments.getAllSyncItemsCount();    
        
    }
    else if (wcscmp(TASKS, sourceName) == 0) {
        p1 = sourceTasks.getNewSyncItems();
        p2 = sourceTasks.getUpdatedSyncItems();
        p3 = sourceTasks.getDeletedSyncItems();
        p4 = sourceTasks.getAllSyncItems();
        
        newCount = sourceTasks.getNewSyncItemsCount();
        modCount = sourceTasks.getUpdatedSyncItemsCount();
        delCount = sourceTasks.getDeletedSyncItemsCount();
        allCount = sourceTasks.getAllSyncItemsCount();    
        
    }
    else if (wcscmp(NOTES, sourceName) == 0) {
        p1 = sourceNotes.getNewSyncItems();
        p2 = sourceNotes.getUpdatedSyncItems();
        p3 = sourceNotes.getDeletedSyncItems();
        p4 = sourceNotes.getAllSyncItems();
        
        newCount = sourceNotes.getNewSyncItemsCount();
        modCount = sourceNotes.getUpdatedSyncItemsCount();
        delCount = sourceNotes.getDeletedSyncItemsCount();
        allCount = sourceNotes.getAllSyncItemsCount();    
        
    }
        
    safeDelete((void**)p1, newCount);
    if (p1 != NULL) {
    delete [] p1;
    }
    safeDelete((void**)p2, modCount);
    if (p2 != NULL) {
            delete [] p2;
    }
    
    safeDelete((void**)p3, delCount);
    if (p3 != NULL) {
            delete [] p3;
    }
    
    safeDelete((void**)p4, allCount);
    if (p4 != NULL) {
            delete [] p4;
    }    

}


int startSync(SyncManager* syncManager, wchar_t* sourceName, wchar_t* isSlow, string &returnCompleteMessage) {

    ActionLOG.debug(TEXT("Enter method startSync."));
    wchar_t localTemp[DIM_LOCAL_TEMP]; localTemp[0] = 0;
    unsigned int ret = 0;
    int syncMode = -1, syncRes = 0;

    // this int represents  the type folder object of pocketpcApplication
    int dataType = 0;
    
    if (wcscmp(CONTACTS, sourceName) == 0) {        
        dataType = olFolderContacts ;   
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0){
        dataType = olFolderCalendar;
    }    
    else if (wcscmp(TASKS, sourceName) == 0){
        dataType = olFolderTasks;
    }  
    else if (wcscmp(NOTES, sourceName) == 0){
        dataType = olFolderNotes;
    }

    if (syncManager == NULL) {          
        error();
        error(returnCompleteMessage);
        return -1;        
    }
    
    ActionLOG.debug(TEXT("Method startSync: preparing sync..."));
    
    if (wcscmp(isSlow, IS_SLOW) == 0) {   
        //MessageBox (NULL, TEXT("Client says is a SLOW SYNC. set preferred mode"), TEXT ("dbg"), MB_SETFOREGROUND |MB_OK);

        if (wcscmp(CONTACTS, sourceName) == 0) 
            sourceContacts.setPreferredSyncMode(SYNC_SLOW);
        else if (wcscmp(APPOINTMENTS, sourceName) == 0)
            sourceAppointments.setPreferredSyncMode(SYNC_SLOW); 
        else if (wcscmp(TASKS, sourceName) == 0)
            sourceTasks.setPreferredSyncMode(SYNC_SLOW);    
        else if (wcscmp(NOTES, sourceName) == 0)
            sourceNotes.setPreferredSyncMode(SYNC_SLOW);   
    } else {
        //MessageBox (NULL, TEXT("Client says is a FAST SYNC"), TEXT ("dbg"), MB_SETFOREGROUND |MB_OK);
    }
       

    if (wcscmp(CONTACTS, sourceName) == 0) {
        ret = syncManager->prepareSync(sourceContacts);
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0) {
        ret = syncManager->prepareSync(sourceAppointments);
    }
    else if (wcscmp(TASKS, sourceName) == 0) {
        ret = syncManager->prepareSync(sourceTasks);
    }
    else if (wcscmp(NOTES, sourceName) == 0) {
        ret = syncManager->prepareSync(sourceNotes);
    }
    
    if (ret != 0) {
        switch (ret) {
            case ERR_PROTOCOL_ERROR:
                LOG.error(MSG_ERR_PROTOCOL_ERROR );
                error(returnCompleteMessage, ERR_PROTOCOL_ERROR, MSG_ERR_PROTOCOL_ERROR );
                goto finally;
                break;

            case ERR_AUTH_NOT_AUTHORIZED:
            case ERR_AUTH_REQUIRED:
                LOG.error(MSG_ERR_AUTH_REQUIRED);
                error(returnCompleteMessage, ERR_AUTH_REQUIRED, MSG_ERR_AUTH_REQUIRED);
                goto finally;
                break;

            case ERR_AUTH_EXPIRED:
                LOG.error(MSG_ERR_AUTH_EXPIRED);
                error(returnCompleteMessage, ERR_AUTH_EXPIRED, MSG_ERR_AUTH_EXPIRED);
                goto finally;
                break;

            case ERR_SRV_FAULT:
                LOG.error(MSG_ERR_SRV_FAULT);
                error(returnCompleteMessage, ERR_SRV_FAULT ,MSG_ERR_SRV_FAULT);
                goto finally; 
                break;

            case ERR_NOT_FOUND:                
                wsprintf(logmsg, MSG_ERR_NOT_FOUND, sourceName);
                LOG.error(logmsg);
                error(returnCompleteMessage, ERR_NOT_FOUND, logmsg);
                goto finally;                
                break;

            default:
                error();
                error(returnCompleteMessage);
                goto finally;
                break;
        }

        //return ret;
    }
    
    ActionLOG.debug(TEXT("Method synchronize: getSyncMode..."));    

    if (wcscmp(CONTACTS, sourceName) == 0) 
            syncMode = sourceContacts.getSyncMode();
    else if (wcscmp(APPOINTMENTS, sourceName) == 0)
            syncMode = sourceAppointments.getSyncMode();
    else if (wcscmp(TASKS, sourceName) == 0)
            syncMode = sourceTasks.getSyncMode();
    else if (wcscmp(NOTES, sourceName) == 0)
            syncMode = sourceNotes.getSyncMode();

    //wsprintf(localTemp, TEXT("Server says %i"), syncMode);
    //MessageBox (NULL, localTemp, TEXT ("dbg"), MB_SETFOREGROUND |MB_OK);

finally:
    
    if (ret != 0) {
        deleteObject(sourceName);
    }
    return syncMode;

}


/*
* method called to manage sync. There are 2 syncSource to manage depending by dataType
*/

int synchronize(SyncManager* syncManager, wchar_t* sourceName, wchar_t* isSlow, string &returnCompleteMessage, LPCSTR xmlAll,
                                LPCSTR xmlNew, LPCSTR xmlMod, LPCSTR xmlDel, wchar_t* path) {

    
    int syncMode = 0, syncRes = 0;
    
    // this int represents  the type folder object of pocketpcApplication
    int dataType = 0;
    
    if (wcscmp(CONTACTS, sourceName) == 0) {        
        dataType = olFolderContacts ;   
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0){
        dataType = olFolderCalendar;
    }    
    else if (wcscmp(TASKS, sourceName) == 0){
        dataType = olFolderTasks;
    }  
    else if (wcscmp(NOTES, sourceName) == 0){
        dataType = olFolderNotes;
    }

    if (syncManager == NULL) {          
        error();
        error(returnCompleteMessage);
        return -1;        
    }

    if (wcscmp(isSlow, IS_SLOW) == 0) {   
        syncMode = 201;
    } else {
        syncMode = 200;
    }

    switch (syncMode) {
                
        case SYNC_SLOW:
            
            ActionLOG.debug(TEXT("Method synchronize: starting slow sync"));

            if (wcscmp(CONTACTS, sourceName) == 0) 
                setAllItems(sourceContacts, dataType, isSlow, xmlAll); 
                //setAllItemsFromFileSystem (sourceContacts, path); 
            else if (wcscmp(APPOINTMENTS, sourceName) == 0)
                setAllItems(sourceAppointments, dataType, isSlow, xmlAll);     
            else if (wcscmp(TASKS, sourceName) == 0)
                setAllItems(sourceTasks, dataType, isSlow, xmlAll);         
            else if (wcscmp(NOTES, sourceName) == 0)
                setAllItems(sourceNotes, dataType, isSlow, xmlAll);         
            break;

        case SYNC_TWO_WAY:      // if is slow sync setAllItems            
            
            
            ActionLOG.debug(TEXT("Method synchronize: is_two-way"));
            if (wcscmp(CONTACTS, sourceName) == 0) 
                setModifiedItems(sourceContacts, dataType, isSlow, xmlNew, xmlMod, xmlDel);                     
            else if (wcscmp(APPOINTMENTS, sourceName) == 0) 
                setModifiedItems(sourceAppointments, dataType, isSlow, xmlNew, xmlMod, xmlDel);         
            else if (wcscmp(TASKS, sourceName) == 0) 
                setModifiedItems(sourceTasks, dataType, isSlow, xmlNew, xmlMod, xmlDel);                    
            else if (wcscmp(NOTES, sourceName) == 0) 
                setModifiedItems(sourceNotes, dataType, isSlow, xmlNew, xmlMod, xmlDel);                    
            break;

 
        case SYNC_NONE:
            break;

        default:
            break;
    }

    if (syncMode == SYNC_NONE) {
        ActionLOG.action(NO_SYNC);
        goto finally;
    }
    
    ActionLOG.debug(TEXT("Method synchronize: Starting sync..."));

    if (wcscmp(CONTACTS, sourceName) == 0) {
            syncRes = syncManager->sync(sourceContacts);
           
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0) {
            syncRes = syncManager->sync(sourceAppointments);
            
    }
    else if (wcscmp(TASKS, sourceName) == 0) {
            syncRes = syncManager->sync(sourceTasks);

    }
    else if (wcscmp(NOTES, sourceName) == 0) {
            syncRes = syncManager->sync(sourceNotes);

    }

    if (syncRes != 0) {        
        error();
        error(returnCompleteMessage, syncRes, lastErrorMsg);        
        return -1;
    }    
    
    ActionLOG.debug(TEXT("Method synchronize: received sync results from server ..."));
    ActionLOG.debug(TEXT("Method synchronize: analysis of returned Items ..."));
        
    if (wcscmp(CONTACTS, sourceName) == 0) 
        manageReturnedItems(sourceContacts, dataType, isSlow, returnCompleteMessage); 
    else if (wcscmp(APPOINTMENTS, sourceName) == 0)
        manageReturnedItems(sourceAppointments, dataType, isSlow, returnCompleteMessage); 
    else if (wcscmp(TASKS, sourceName) == 0)
        manageReturnedItems(sourceTasks, dataType, isSlow, returnCompleteMessage); 
    else if (wcscmp(NOTES, sourceName) == 0)
        manageReturnedItems(sourceNotes, dataType, isSlow, returnCompleteMessage); 
    
    if (dbg) {
            MessageBox (NULL, (wchar_t*)returnCompleteMessage, TEXT ("completeMessage"), MB_SETFOREGROUND |MB_OK);
    }
                

finally:
    ActionLOG.debug(TEXT("Method synchronize: delete source.getNewSyncItemsCount, source.getNewSyncItemsCount..."));
    deleteObject(sourceName);

    ActionLOG.debug(TEXT("Exit Method synchronize."));
    lastErrorCode = 0;
    return 0;
}

/*
* for debug only
*/ 
void displayMappings(SyncSource& s) {
    unsigned int n = 0;
    SyncMap** map;

    n = s.getMapSize();
    map = s.getLUIDGUIDMapping();
    
    for (unsigned int i=0; i<n; ++i) {
        //wsprintf(logmsg, TEXT("luid: %s, guid: %s"), map[i]->getLUID(NULL), map[i]->getGUID(NULL));
        //LOG.info(logmsg);
    }
}


void setMappings(SyncSource& s, wchar_t* xmlMapping) {
   
    ActionLOG.debug(TEXT("Enter Method setMappings."));
    unsigned int n = 0, pos = 0;    
    wchar_t* element = NULL;
    wchar_t* tmp     = NULL;

    element = getElementContent(xmlMapping, TEXT("numItems"), &pos);
        
    if (element == NULL) {
        if (dbg) {          
        MessageBox (NULL, TEXT("no item to map..."), TEXT(""), MB_SETFOREGROUND |MB_OK);    
    }
    ActionLOG.debug(TEXT("Method setMappings. No items to map"));
    return;
    }
        // swscanf(dataString, TEXT("%d/%d/%d %d:%d:%d"), &t.wDay, &k, &t.wYear, &t.wHour, &t.wMinute, &t.wSecond);
    n = wcstol(element, &tmp, 0);       
   
    SyncMap**  mappings = NULL;
    ActionLOG.debug(TEXT("Method setMappings. Start to map..."));   
    if (n > 0) {
            
        mappings = new SyncMap* [n];
        for (unsigned int i=0; i<n; ++i) {
                mappings[i] = NULL; // every map pointer initialized to null!
        }
    }

    wchar_t luid[DIM_KEY];  
    wchar_t guid[DIM_KEY]; 
    pos = 0;

    for (unsigned int i = 0; i<n; ++i) {
                
        unsigned int tmpPos = 0;

        //localUid
        element = getElementContent(&xmlMapping[pos], TEXT("luid"), &tmpPos);           
        wsprintf(luid, element);

        pos = tmpPos + pos;

        //Guid
        element = getElementContent(&xmlMapping[pos], TEXT("guid"), &tmpPos);   
        wsprintf(guid, element);
                
        mappings[i] = new SyncMap(guid, luid);

    }
    if (n > 0) {
        s.setLUIDGUIDMapping(mappings, n);
    }
    ActionLOG.debug(TEXT("Method setMappings. End map..."));
    ActionLOG.debug(TEXT("Exit Method setMappings."));

}


int WINAPI endSynchronization(LPCSTR xmlMapping, LPCSTR folderType) {
        
    ActionLOG.debug(TEXT("Enter Method endSynchronization."));

    int ret = 0;    
    long l = strlen(xmlMapping);
    
    wchar_t* wxmlMapping = new wchar_t[l + 1];
    MultiByteToWideChar (CP_ACP, 0, xmlMapping, l, wxmlMapping, l);
    wxmlMapping[l] = 0;     

    wchar_t wfolderType[DIM_LOCAL_TEMP];        
    l = strlen(folderType);
    MultiByteToWideChar (CP_ACP, 0, folderType, l, wfolderType, l);
    wfolderType[l] = 0;

    if (dbg) {
        MessageBox (NULL, wxmlMapping, TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
        MessageBox (NULL, TEXT("Mapping..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
    }
    ActionLOG.debug(TEXT("Method endSynchronization. To do setMappings"));

    if (wcscmp(wfolderType, OL_FOLDER_CONTACTS) == 0) {

        setMappings(sourceContacts, wxmlMapping);
        
        
        
        if (syncManager->endSync(sourceContacts) != 0) {
            if (dbg) 
                MessageBox (NULL, TEXT("Error..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
            error();                        
            ret =  -1;
        }
        // Release mapping
        
        SyncMap** sm;
        sm = sourceContacts.getLUIDGUIDMapping();
        safeDelete((void**)sm, sourceContacts.getMapSize());
        

    } else if (wcscmp(wfolderType, OL_FOLDER_APPOINTMENTS) == 0){
        setMappings(sourceAppointments, wxmlMapping);
        
        displayMappings(sourceAppointments);
        
        
        
        if (syncManager->endSync(sourceAppointments) != 0) {
            if (dbg) 
                MessageBox (NULL, TEXT("Error..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);                   
            error();
            ret =  -1;
        }               
        // Release mapping
        
        SyncMap** sm;
        sm = sourceAppointments.getLUIDGUIDMapping();
        safeDelete((void**)sm, sourceAppointments.getMapSize());
            

    } else if (wcscmp(wfolderType, OL_FOLDER_TASKS) == 0){
        setMappings(sourceTasks, wxmlMapping);
        
        displayMappings(sourceTasks);
        
        

        if (syncManager->endSync(sourceTasks) != 0) {
            if (dbg) 
                MessageBox (NULL, TEXT("Error..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);                   
            error();
            ret =  -1;
        }               
        // Release mapping
        
        SyncMap** sm;
        sm = sourceTasks.getLUIDGUIDMapping();
        safeDelete((void**)sm, sourceTasks.getMapSize());
        
    } else if (wcscmp(wfolderType, OL_FOLDER_NOTES) == 0){
        setMappings(sourceNotes, wxmlMapping);
        
        displayMappings(sourceNotes);
        
        

        if (syncManager->endSync(sourceNotes) != 0) {
            if (dbg) 
                MessageBox (NULL, TEXT("Error..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);                   
            error();
            ret =  -1;
        }               
        // Release mapping
        
        SyncMap** sm;
        sm = sourceNotes.getLUIDGUIDMapping();
        safeDelete((void**)sm, sourceNotes.getMapSize());
            
    }


    lastErrorCode = 0;
    ActionLOG.debug(TEXT("Method endSynchronization. Release mappings."));

    if (dbg) 
        MessageBox (NULL, TEXT("No Error..."), TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
    
    if (wxmlMapping != NULL)
        delete wxmlMapping;
    // if (syncManager != NULL)
    //    delete syncManager;
    ActionLOG.debug(TEXT("Method endSynchronization. Delete variables."));
        
    // renew sources...
    sourceContacts = SyncSource(CONTACTS);
    sourceAppointments = SyncSource(APPOINTMENTS);
    sourceTasks = SyncSource(TASKS);
    sourceNotes = SyncSource(NOTES);
    
    ActionLOG.debug(TEXT("Exit Method endSynchronization. "));
    
    ActionLOG.action(SYNC_ENDED);
        
    return ret;


}

void WINAPI closeLog() {
    
    #if defined( ACTION_DEBUG ) || defined( DEBUGGER )
        ActionLog::close();
    #endif

}

void WINAPI openLog(LPCSTR pathVB) {
    int l;
    wchar_t path[DIM_LOCAL_TEMP];       
    l = strlen(pathVB);
    MultiByteToWideChar (CP_ACP, 0, pathVB, l, path, l);
    path[l] = 0;
    
    wchar_t value [128];
    value[0] = 0;

    OutlookConfig oc = OutlookConfig();
    oc.getClientConfigurationOutlook(TEXT(""), PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL, value);
        
    if (wcscmp(value, TEXT("0")) == 0) {
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = FALSE;    
    } else if (wcscmp(value, TEXT("2")) == 0) {
        Log::debugEnabled = TRUE;    
        Log::infoEnabled = TRUE;    
    } else { // default info
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = TRUE;    
    }

    ActionLog::setActionPath(path);
    
    #if defined( ACTION_DEBUG ) || defined( DEBUGGER )
        ActionLog::open(true);
    #endif

    /*
    #ifdef DEBUG
        Log::debugEnabled = TRUE;
    #else
        Log::debugEnabled = FALSE;
    #endif

   */

    #ifdef ACTION_DEBUG
        ActionLog::actionDebugEnabled = TRUE;
    #else
        ActionLog::actionDebugEnabled = FALSE;
    #endif

    #ifdef DEBUGGER
        ActionLog::debugEnabled = TRUE;        
    #else
        ActionLog::debugEnabled = FALSE;
    #endif
    

}

void WINAPI createSyncManager() {
   
    SyncManagerFactory factory = SyncManagerFactory();   
    syncManager = factory.getSyncManager(APPLICATION_URI);
   
}

void WINAPI deleteSyncManager() {
    if (syncManager != NULL) {
        delete syncManager;
    }
}



BSTR WINAPI startSynchronization (LPCSTR isSlowVB, LPCSTR folderType, LPCSTR pathVB) {
    
    /******** INIT ************/
    
    wchar_t isSlow[DIM_LOCAL_TEMP];     
    int l = strlen(isSlowVB);
    MultiByteToWideChar (CP_ACP, 0, isSlowVB, l, isSlow, l);
    isSlow[l] = 0;   
        
    wchar_t wfolderType[DIM_LOCAL_TEMP];        
    l = strlen(folderType);
    MultiByteToWideChar (CP_ACP, 0, folderType, l, wfolderType, l);
    wfolderType[l] = 0;    
   
    wchar_t path[DIM_LOCAL_TEMP];       
    l = strlen(pathVB);
    MultiByteToWideChar (CP_ACP, 0, pathVB, l, path, l);
    path[l] = 0;
    
    //
    int rc = 0;
    lastErrorCode = 0;
    
    string returnCompleteMessage = string();
    ActionLOG.debug(TEXT("Enter Method synchronization."));
        
    /*********************************/
     // Initialization and creation SyncManager
    //SyncManagerFactory factory = SyncManagerFactory();   
    //syncManager = factory.getSyncManager(APPLICATION_URI);
        
    if (wcscmp(wfolderType, OL_FOLDER_CONTACTS) == 0) { //is contact
                
        ActionLOG.debug(TEXT("Method synchronization: contacts"));        
        rc = startSync(syncManager, CONTACTS, isSlow, returnCompleteMessage);

        if (dbg) {
            MessageBox (NULL, TEXT("In Contacts"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, returnCompleteMessage, TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);                       
        }

    }
    else if (wcscmp(wfolderType, OL_FOLDER_APPOINTMENTS) == 0){ //is appointments
        ActionLOG.debug(TEXT("Method synchronization: calendar"));
        // rc = synchronize(syncManager, APPOINTMENTS, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel);
        rc = startSync(syncManager, APPOINTMENTS, isSlow, returnCompleteMessage);
        if (dbg) {
            MessageBox (NULL, TEXT("In Appointments"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    }
    else if (wcscmp(wfolderType, OL_FOLDER_TASKS) == 0){ //is tasks
        ActionLOG.debug(TEXT("Method synchronization: tasks"));
            // rc = synchronize(syncManager, TASKS, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel);
            rc = startSync(syncManager, TASKS, isSlow, returnCompleteMessage);
            if (dbg) {
                MessageBox (NULL, TEXT("In Task"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }
    else if (wcscmp(wfolderType, OL_FOLDER_NOTES) == 0){ //is notes
        ActionLOG.debug(TEXT("Method synchronization: tasks"));
        // rc = synchronize(syncManager, NOTES, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel);
        rc = startSync(syncManager, NOTES, isSlow, returnCompleteMessage);
        if (dbg) {
            MessageBox (NULL, TEXT("In Notes"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    }
    else {
            //nothing to do
    }   
    
    if (rc == 200) {
        returnCompleteMessage = TEXT("");
        returnCompleteMessage = returnCompleteMessage + TEXT("FAST");
    } else if (rc == 201) {
        returnCompleteMessage = TEXT("");
        returnCompleteMessage = returnCompleteMessage + TEXT("SLOW");
    }


    int lung = length(returnCompleteMessage);
    
    
    char* vbRetErrorMsg = NULL;
    vbRetErrorMsg = new char[lung + 1];
    
    BOOL usedDefaultChar;
    WideCharToMultiByte (CP_ACP, 0, (wchar_t*)returnCompleteMessage, lung, 
                    vbRetErrorMsg, lung, "?", &usedDefaultChar);            
    vbRetErrorMsg[lung] = 0;

    bstr = SysAllocStringByteLen(vbRetErrorMsg, strlen(vbRetErrorMsg));     
    delete vbRetErrorMsg;
    ActionLOG.debug(TEXT("Method synchronization: created string to exchange to VB"));

    ActionLOG.debug(TEXT("Exit method synchronization."));

    return bstr;


}


/*  entry point from VB application. 
*   Accept and manage Variant array of String, long and int. 
*   The entry array is a String array in which elements rapresent SyncSource name.
*   Now accepted and managed element names are "contacts" and "appointments"
*   Array contains at least one element
*/
BSTR WINAPI synchronization (LPCSTR isSlowVB, LPCSTR xmlAll, 
                             LPCSTR xmlNew, LPCSTR xmlMod, 
                             LPCSTR xmlDel, LPCSTR folderType, LPCSTR pathVB) {
        
    
    wchar_t isSlow[DIM_LOCAL_TEMP];     
    int l = strlen(isSlowVB);
    MultiByteToWideChar (CP_ACP, 0, isSlowVB, l, isSlow, l);
    isSlow[l] = 0;

    wchar_t path[DIM_LOCAL_TEMP];       
    l = strlen(pathVB);
    MultiByteToWideChar (CP_ACP, 0, pathVB, l, path, l);
    path[l] = 0;

    wchar_t wfolderType[DIM_LOCAL_TEMP];        
    l = strlen(folderType);
    MultiByteToWideChar (CP_ACP, 0, folderType, l, wfolderType, l);
    wfolderType[l] = 0;
    
    

    //
    int rc = 0;
    lastErrorCode = 0;
    
    string returnCompleteMessage = string();
    ActionLOG.debug(TEXT("Enter Method synchronization."));

    /*********************************/
                                            
    if (wcscmp(wfolderType, OL_FOLDER_CONTACTS) == 0) { //is contact
                
        ActionLOG.debug(TEXT("Method synchronization: contacts"));
        rc = synchronize(syncManager, CONTACTS, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel, path);
        
        if (dbg) {
            MessageBox (NULL, TEXT("In Contacts"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, returnCompleteMessage, TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);                       
        }

    }
    else if (wcscmp(wfolderType, OL_FOLDER_APPOINTMENTS) == 0){ //is appointments
        ActionLOG.debug(TEXT("Method synchronization: calendar"));
        rc = synchronize(syncManager, APPOINTMENTS, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel, path);

        if (dbg) {
            MessageBox (NULL, TEXT("In Appointments"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    }
    else if (wcscmp(wfolderType, OL_FOLDER_TASKS) == 0){ //is tasks
        ActionLOG.debug(TEXT("Method synchronization: tasks"));
            rc = synchronize(syncManager, TASKS, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel, path);

            if (dbg) {
                MessageBox (NULL, TEXT("In Task"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }
    else if (wcscmp(wfolderType, OL_FOLDER_NOTES) == 0){ //is notes
        ActionLOG.debug(TEXT("Method synchronization: tasks"));
        rc = synchronize(syncManager, NOTES, isSlow, returnCompleteMessage, xmlAll, xmlNew, xmlMod, xmlDel, path);

        if (dbg) {
            MessageBox (NULL, TEXT("In Notes"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    }
    else {
            //nothing to do
    }   
    
    ActionLOG.debug(TEXT("Method synchronization: prepare string to exchange to VB"));

    int lung = length(returnCompleteMessage);
    
    
    char* vbRetErrorMsg = NULL;
    vbRetErrorMsg = new char[lung + 1];
    
    BOOL usedDefaultChar;
    WideCharToMultiByte (CP_ACP, 0, (wchar_t*)returnCompleteMessage, lung, 
                    vbRetErrorMsg, lung, "?", &usedDefaultChar);            
    vbRetErrorMsg[lung] = 0;

    bstr = SysAllocStringByteLen(vbRetErrorMsg, strlen(vbRetErrorMsg));     
    delete vbRetErrorMsg;
    ActionLOG.debug(TEXT("Method synchronization: created string to exchange to VB"));

    ActionLOG.debug(TEXT("Exit method synchronization."));

    return bstr;

}



/*
*
*
* CONFIGURATION
*
*
*/

DWORD WINAPI freeMemAlloc() {   
    //SysFreeString(bstr);
    return 0;
}


DWORD WINAPI getServerConfigParameter (LPCSTR propertyName, LPSTR propertyValue, LONG propValueSize) {

    int l = 0;
    // trasnform propertyName in wchar_t
    wchar_t wpropertyName[DIM_LOCAL_TEMP];      
    l = strlen(propertyName);
    MultiByteToWideChar (CP_ACP, 0, propertyName, l, wpropertyName, l);
    wpropertyName[l] = 0;
    
    wchar_t wpropertyValue[DIM_LOCAL_TEMP];     
    
    OutlookConfig oc = OutlookConfig();
    oc.getConfigParameter(wpropertyName, wpropertyValue);
            
    BOOL usedDefaultChar;
    char internalBuffer[DIM_LOCAL_TEMP];
    WideCharToMultiByte (CP_ACP, 0, wpropertyValue, DIM_RET_ERROR_MESSAGE -1, internalBuffer, DIM_RET_ERROR_MESSAGE -1, "?", &usedDefaultChar);     

    unsigned long size = propValueSize;

    if (size > strlen(internalBuffer) ) {
            strcpy(propertyValue, internalBuffer);  
            propertyValue[strlen(propertyValue)] = ' ';
    }               
    return 0;

}

DWORD WINAPI setServerConfigParameters (LPCSTR syncURL, LPCSTR serverName, LPCSTR username, LPCSTR password,  LPCSTR deviceId ) {
        
    int l = 0;
    wchar_t werrorMsg[DIM_LOCAL_TEMP];

    //************ syncURL************/
    wchar_t wsyncURL[DIM_LOCAL_TEMP];         
    wsprintf(wsyncURL,TEXT(""));
    if (syncURL != NULL)
        l = strlen(syncURL);
    if (l > 0) {
        MultiByteToWideChar (CP_ACP, 0, syncURL, l, wsyncURL, l);
        wsyncURL[l] = 0;
    }

    //************ serverName ************/
    wchar_t wserverName[DIM_LOCAL_TEMP];    
    wsprintf(wserverName, TEXT(""));
    if (serverName != NULL)
        l = strlen(serverName);
    if (l > 0) {
        MultiByteToWideChar (CP_ACP, 0, serverName, l, wserverName, l);
        wserverName[l] = 0;
    }
    //************ username ************/
    wchar_t wusername[DIM_LOCAL_TEMP];       
    wsprintf(wusername, TEXT(""));
    if (username != NULL)
        l = strlen(username);
    if (l > 0) {
        MultiByteToWideChar (CP_ACP, 0, username, l, wusername, l);
        wusername[l] = 0;
    }
    //************ password ************/
    wchar_t wpassword[DIM_LOCAL_TEMP];          
    wsprintf(wpassword,TEXT(""));
    if (password != NULL)
        l = strlen(password);
    if (l > 0) {
        MultiByteToWideChar (CP_ACP, 0, password, l, wpassword, l);
        wpassword[l] = 0;
    }
    //************ deviceId ************/
    wchar_t wdeviceId[DIM_LOCAL_TEMP];          
    wsprintf(wdeviceId, TEXT(""));
    if (deviceId != NULL)
        l = strlen(deviceId);
    if (l > 0) {
        MultiByteToWideChar (CP_ACP, 0, deviceId, l, wdeviceId, l);
        wdeviceId[l] = 0;
    }
    OutlookConfig oc = OutlookConfig();

    int ret = oc.setConfigParameters(wsyncURL, wserverName, wusername, wpassword, wdeviceId, werrorMsg);

    return ret;
                
}


DWORD WINAPI setSourceSync (LPCSTR sourceName, LPCSTR propertyName, LPCSTR propertyValue) {

    int l = 0;
    int ret = 0;
    wchar_t result [DIM_LOCAL_TEMP];
    wchar_t wsourceName[DIM_LOCAL_TEMP];        
    l = strlen(sourceName);
    MultiByteToWideChar (CP_ACP, 0, sourceName, l, wsourceName, l);
    wsourceName[l] = 0;
    
    wchar_t wpropertyValue[DIM_LOCAL_TEMP];     
    l = strlen(propertyValue);
    MultiByteToWideChar (CP_ACP, 0, propertyValue, l, wpropertyValue, l);
    wpropertyValue[l] = 0;

    wchar_t wpropertyName[DIM_LOCAL_TEMP];          
    l = strlen(propertyName);
    MultiByteToWideChar (CP_ACP, 0, propertyName, l, wpropertyName, l);
    wpropertyName[l] = 0;
    
    OutlookConfig oc = OutlookConfig();

    ret = oc.setSourceParameter(wsourceName, wpropertyName, wpropertyValue, result);

    return ret;
                
}

DWORD WINAPI getSourceSync (LPCSTR sourceName, LPSTR propertyName, LPSTR propertyValue, LONG propValueSize) {
    int l = 0;

    
    // trasnform propertyName in wchar_t
    wchar_t wsourceName[DIM_LOCAL_TEMP];        
    l = strlen(sourceName);
    MultiByteToWideChar (CP_ACP, 0, sourceName, l, wsourceName, l);
    wsourceName[l] = 0;
    
    wchar_t wpropertyName[DIM_LOCAL_TEMP];          
    l = strlen(propertyName);
    MultiByteToWideChar (CP_ACP, 0, propertyName, l, wpropertyName, l);
    wpropertyName[l] = 0;

    wchar_t wpropertyValue[DIM_LOCAL_TEMP];     
    OutlookConfig oc = OutlookConfig();
    oc.getSourceParameter(wsourceName, wpropertyName, wpropertyValue);
            
    BOOL usedDefaultChar;
    char internalBuffer[DIM_LOCAL_TEMP];
    WideCharToMultiByte (CP_ACP, 0, wpropertyValue, DIM_LOCAL_TEMP -1, internalBuffer, DIM_LOCAL_TEMP -1, "?", &usedDefaultChar);   

    unsigned long size = propValueSize;

    if (size > strlen(internalBuffer) ) {
        strcpy(propertyValue, internalBuffer);  
        propertyValue[strlen(propertyValue)] = ' ';
    }
    return 0;

}

DWORD WINAPI setClientConfiguration (LPCSTR sourceName, LPCSTR propertyName, LPCSTR propertyValue) {

    int l = 0;
    int ret = 0;

    wchar_t wsourceName[DIM_LOCAL_TEMP];        
    l = strlen(sourceName);
    MultiByteToWideChar (CP_ACP, 0, sourceName, l, wsourceName, l);
    wsourceName[l] = 0;
    
    wchar_t wpropertyValue[DIM_LOCAL_TEMP];     
    l = strlen(propertyValue);
    MultiByteToWideChar (CP_ACP, 0, propertyValue, l, wpropertyValue, l);
    wpropertyValue[l] = 0;

    wchar_t wpropertyName[DIM_LOCAL_TEMP];          
    l = strlen(propertyName);
    MultiByteToWideChar (CP_ACP, 0, propertyName, l, wpropertyName, l);
    wpropertyName[l] = 0;
    
    OutlookConfig oc = OutlookConfig();

    ret = oc.setClientConfigurationOutlook(wsourceName, wpropertyName, wpropertyValue);

    return ret;
                
}

DWORD WINAPI getClientConfiguration (LPCSTR sourceName, LPSTR propertyName, 
                                     LPSTR propertyValue, LONG propValueSize) {    

    int l = 0;
    int ret = 0;

    wchar_t wsourceName[DIM_LOCAL_TEMP];        
    l = strlen(sourceName);
    MultiByteToWideChar (CP_ACP, 0, sourceName, l, wsourceName, l);
    wsourceName[l] = 0;
    
    wchar_t wpropertyValue[DIM_LOCAL_TEMP];     
    l = strlen(propertyValue);
    MultiByteToWideChar (CP_ACP, 0, propertyValue, l, wpropertyValue, l);
    wpropertyValue[l] = 0;

    wchar_t wpropertyName[DIM_LOCAL_TEMP];          
    l = strlen(propertyName);
    MultiByteToWideChar (CP_ACP, 0, propertyName, l, wpropertyName, l);
    wpropertyName[l] = 0;
    
    OutlookConfig oc = OutlookConfig();

    ret = oc.getClientConfigurationOutlook(wsourceName, wpropertyName, wpropertyValue);

    BOOL usedDefaultChar;
    char internalBuffer[DIM_LOCAL_TEMP];
    WideCharToMultiByte (CP_ACP, 0, wpropertyValue, DIM_RET_ERROR_MESSAGE -1, internalBuffer, DIM_RET_ERROR_MESSAGE -1, "?", &usedDefaultChar);     
    
    unsigned long size = propValueSize;

    if (size > strlen(internalBuffer) ) {
        strcpy(propertyValue, internalBuffer);  
        propertyValue[strlen(propertyValue)] = ' ';
    }
    return ret;
}



DWORD WINAPI getDayLight () {
        
    TIME_ZONE_INFORMATION tmz;

    GetTimeZoneInformation(&tmz);
    long ret = 0;
    ret = tmz.DaylightBias;
    
    if (ret = -60)
        ret = 1;

    return ret;
}

DWORD WINAPI isAnotherWindow() {
    HWND hwnd = FindWindow(NULL, TEXT("SyncClient Outlook"));
    if (hwnd != NULL) {
        SetForegroundWindow(hwnd);       
        CloseHandle(hwnd);      
        return 1;
    }
    return 0;
}